@@ -31,28 +31,57 @@
 
 #define EMPTY -1000
 
+// A pointer to the JVM for callbacks
+static JavaVM *jvm;
+static void execJavaLogCallback(jobject cb, int level, const char *message)
+{
+  JNIEnv *env;
+  jint rs;
+  jstring jstrMsg;
+  jclass cls;
+  jmethodID mid;
+  if ((jvm == NULL) || (cb == NULL))
+    return;
+  rs = (*jvm)->AttachCurrentThread(jvm, (void **)&env, NULL);
+  if (rs != JNI_OK)
+    return;
+  cls = (*env)->GetObjectClass(env, cb);
+  if (cls == NULL)
+    return;
+  mid = (*env)->GetMethodID(env, cls, "logMessage", "(ILjava/lang/String;)V");
+  if (mid == NULL)
+    return;
+  jstrMsg = (*env)->NewStringUTF(env, message);
+  (*env)->CallVoidMethod(env, cb, mid, level, jstrMsg);
+}
 /*
- * Class:     org_liblouis_liblouisutdml
+ * Class:     org_liblouis_LibLouisUTDML
  * Method:    initialize
  * Signature: (Ljava/lang/String;Ljava/lang/String;Ljava/lang/String;)V
  */
-JNIEXPORT void JNICALL Java_org_liblouis_liblouisutdml_initialize
+JNIEXPORT void JNICALL Java_org_liblouis_LibLouisUTDML_initialize
   (JNIEnv *env, jclass class, jstring dataPath, jstring
 writeablePath, jstring logfile)
 {
   const jbyte *dataPathX = NULL;
   const jbyte *writeablePathX = NULL;
   const jbyte *logfileX = NULL;
+  char *tmpDataPath;
   dataPathX = (*env)->GetStringUTFChars (env, dataPath, NULL);
   if (dataPathX == NULL)
     goto release;
   writeablePathX = (*env)->GetStringUTFChars (env, writeablePath, NULL);
   if (writeablePathX == NULL)
     goto release;
-  logfileX = (*env)->GetStringUTFChars (env, logfile, NULL);
-  if (logfileX == NULL)
-    goto release;
-  lou_setDataPath (dataPathX);
+  if (logfile != NULL)
+  {
+    logfileX = (*env)->GetStringUTFChars (env, logfile, NULL);
+    if (logfileX == NULL)
+      goto release;
+  }
+  tmpDataPath = strdup(dataPathX);
+  lou_setDataPath (tmpDataPath);
+  free(tmpDataPath);
   lbu_setWriteablePath (writeablePathX);
   read_configuration_file (NULL, logfileX, NULL, 0);
 release:
@@ -65,23 +94,36 @@ release:
 }
 
 /*
- * Class:     org_liblouis_liblouisutdml
+ * Class:     org_liblouis_LibLouisUTDML
  * Method:    version
  * Signature: ()Ljava/lang/String;
  */
-JNIEXPORT jstring JNICALL Java_org_liblouis_liblouisutdml_version
+JNIEXPORT jstring JNICALL Java_org_liblouis_LibLouisUTDML_version
   (JNIEnv * env, jobject obj)
 {
   return (*env)->NewStringUTF (env, lbu_version ());
 }
 
 /*
- * Class:     org_liblouis_liblouisutdml
+ * Class:     org_liblouis_LibLouisUTDML
+ * Method:    loadXMLCatalog
+ * Signature: (Ljava/lang/String;)V
+ */
+JNIEXPORT void JNICALL Java_org_liblouis_LibLouisUTDML_loadXMLCatalog
+  (JNIEnv *env, jobject this, jstring filename)
+{
+  const char *cFilename = (*env)->GetStringUTFChars(env, filename, 0);
+  lbu_loadXMLCatalog(cFilename);
+  (*env)->ReleaseStringUTFChars(env, filename, cFilename);
+}
+
+/*
+ * Class:     org_liblouis_LibLouisUTDML
  * Method:    translateString
  * Signature: (Ljava/lang/String;[B[B[ILjava/lang/String;Ljava/lang/String;I)Z
  */
 JNIEXPORT jboolean JNICALL
-Java_org_liblouis_liblouisutdml_translateString (JNIEnv * env,
+Java_org_liblouis_LibLouisUTDML_translateString (JNIEnv * env,
 						 jobject obj,
 						 jstring
 						 configFileList,
@@ -164,12 +206,12 @@ release:
 }
 
 /*
- * Class:     org_liblouis_liblouisutdml
+ * Class:     org_liblouis_LibLouisUTDML
  * Method:    backTranslateString
  * Signature: (Ljava/lang/String;[B[B[ILjava/lang/String;Ljava/lang/String;I)Z
  */
 JNIEXPORT jboolean JNICALL
-Java_org_liblouis_liblouisutdml_backTranslateString (JNIEnv * env,
+Java_org_liblouis_LibLouisUTDML_backTranslateString (JNIEnv * env,
 						     jobject obj,
 						     jstring
 						     configFileList,
@@ -201,9 +243,11 @@ Java_org_liblouis_liblouisutdml_backTranslateString (JNIEnv * env,
   if (inbufX == NULL)
     goto release;
   inlen = (*env)->GetArrayLength (env, inbuf);
+  logMessage(LOG_DEBUG, "inlen=%d", inlen);
   if (outbuf == NULL)
     goto release;
   (*env)->GetIntArrayRegion (env, outlen, 0, 1, &outlenX);
+  logMessage(LOG_DEBUG, "outlenX=%d", outlenX);
   if (outlenX == EMPTY)
     goto release;
   if (logFile != NULL)
@@ -225,18 +269,23 @@ Java_org_liblouis_liblouisutdml_backTranslateString (JNIEnv * env,
     {
       int wcLength;
       int utf8Length;
+      logMessage(LOG_DEBUG, "After backTranslate outlenX=%d", outlenX);
       if (ud->format_for == utd)
 	{
+          logMessage(LOG_DEBUG, "Preparing to return UTD");
 	  (*env)->SetByteArrayRegion (env, outbuf, 0, outlenX,
 				      (jbyte *) outbufX);
 	  utf8Length = outlenX;
 	}
       else
 	{
+          logMessage(LOG_DEBUG, "Preparing to return non-UTD");
 	  wcLength = outlenX;
 	  utf8Length = outlenX;
+          outbufY = (jbyte *)malloc(utf8Length);
 	  wc_string_to_utf8 (outbufX, &wcLength, outbufY, &utf8Length);
 	  (*env)->SetByteArrayRegion (env, outbuf, 0, utf8Length, outbufY);
+          free(outbufY);
 	}
       (*env)->SetIntArrayRegion (env, outlen, 0, 1, &utf8Length);
     }
@@ -246,7 +295,7 @@ release:
   if (inbufX != NULL)
     (*env)->ReleaseByteArrayElements (env, inbuf, inbufX, 0);
   if (outbufX != NULL)
-    free (inbufX);
+    free (outbufX);
   if (logf != NULL)
     (*env)->ReleaseStringUTFChars (env, logFile, logf);
   if (settings != NULL)
@@ -255,11 +304,11 @@ release:
 }
 
 /*
- * Class:     org_liblouis_liblouisutdml
+ * Class:     org_liblouis_LibLouisUTDML
  * Method:    translateFile
  * Signature: (Ljava/lang/String;Ljava/lang/String;Ljava/lang/String;Ljava/lang/String;Ljava/lang/String;I)Z
  */
-JNIEXPORT jboolean JNICALL Java_org_liblouis_liblouisutdml_translateFile
+JNIEXPORT jboolean JNICALL Java_org_liblouis_LibLouisUTDML_translateFile
   (JNIEnv * env, jobject obj, jstring configFileList, jstring inputFileName,
    jstring outputFileName, jstring logFile, jstring settingsString, jint mode)
 {
@@ -306,12 +355,12 @@ release:
 }
 
 /*
- * Class:     org_liblouis_liblouisutdml
+ * Class:     org_liblouis_LibLouisUTDML
  * Method:    translateTextFile
  * Signature: (Ljava/lang/String;Ljava/lang/String;Ljava/lang/String;Ljava/lang/String;Ljava/lang/String;I)Z
  */
 JNIEXPORT jboolean JNICALL
-Java_org_liblouis_liblouisutdml_translateTextFile (JNIEnv * env,
+Java_org_liblouis_LibLouisUTDML_translateTextFile (JNIEnv * env,
 						   jobject obj,
 						   jstring
 						   configFileList,
@@ -366,12 +415,12 @@ release:
 }
 
 /*
- * Class:     org_liblouis_liblouisutdml
+ * Class:     org_liblouis_LibLouisUTDML
  * Method:    backTranslateFile
  * Signature: (Ljava/lang/String;Ljava/lang/String;Ljava/lang/String;Ljava/lang/String;Ljava/lang/String;I)Z
  */
 JNIEXPORT jboolean JNICALL
-Java_org_liblouis_liblouisutdml_backTranslateFile (JNIEnv * env,
+Java_org_liblouis_LibLouisUTDML_backTranslateFile (JNIEnv * env,
 						   jobject obj,
 						   jstring
 						   configFileList,
@@ -426,11 +475,11 @@ release:
 }
 
 /*
- * Class:     org_liblouis_liblouisutdml
+ * Class:     org_liblouis_LibLouisUTDML
  * Method:    charToDots
  * Signature: (Ljava/lang/String;[B[BLjava/lang/String;I)V
  */
-JNIEXPORT jboolean JNICALL Java_org_liblouis_liblouisutdml_charToDots
+JNIEXPORT jboolean JNICALL Java_org_liblouis_LibLouisUTDML_charToDots
   (JNIEnv * env, jobject obj, jstring tableList, jbyteArray inbuf,
    jbyteArray outbuf, jstring logFile, jint mode)
 {
@@ -473,11 +522,11 @@ release:
 }
 
 /*
- * Class:     org_liblouis_liblouisutdml
+ * Class:     org_liblouis_LibLouisUTDML
  * Method:    dotsToChar
  * Signature: (Ljava/lang/String;[B[BLjava/lang/String;I)V
  */
-JNIEXPORT jboolean JNICALL Java_org_liblouis_liblouisutdml_dotsToChar
+JNIEXPORT jboolean JNICALL Java_org_liblouis_LibLouisUTDML_dotsToChar
   (JNIEnv * env, jobject obj, jstring tableList, jbyteArray inbuf,
    jbyteArray outbuf, jstring logFile, jint mode)
 {
@@ -519,11 +568,11 @@ release:
 }
 
 /*
- * Class:     org_liblouis_liblouisutdml
+ * Class:     org_liblouis_LibLouisUTDML
  * Method:    checkTable
  * Signature: (Ljava/lang/String;Ljava/lang/String;I)V
  */
-JNIEXPORT jboolean JNICALL Java_org_liblouis_liblouisutdml_checkTable
+JNIEXPORT jboolean JNICALL Java_org_liblouis_LibLouisUTDML_checkTable
   (JNIEnv * env, jobject obj, jstring tableList, jstring logFile, jint mode)
 {
   const jbyte *tableListX = NULL;
@@ -548,11 +597,11 @@ release:
 }
 
 /*
- * Class:     org_liblouis_liblouisutdml
+ * Class:     org_liblouis_LibLouis
  * Method:    compileString
  * Signature: (Ljava/lang/String;Ljava/lang/String;Ljava/lang/String;)Z
  */
-JNIEXPORT jboolean JNICALL Java_org_liblouis_liblouisutdml_compileString
+JNIEXPORT jboolean JNICALL Java_org_liblouis_LibLouis_compileString
   (JNIEnv * env, jobject obj, jstring tableList, jstring newEntry,
    jstring logFile)
 {
@@ -587,11 +636,11 @@ release:
 }
 
 /*
- * Class:     org_liblouis_liblouisutdml
+ * Class:     org_liblouis_LibLouis
  * Method:    setDataPath
  * Signature: (Ljava/lang/String;)V
  */
-JNIEXPORT void JNICALL Java_org_liblouis_liblouisutdml_setDataPath
+JNIEXPORT void JNICALL Java_org_liblouis_LibLouis_setDataPath
   (JNIEnv * env, jclass obj, jstring path)
 {
   const jbyte *pathX = NULL;
@@ -605,22 +654,100 @@ release:
 }
 
 /*
- * Class:     org_liblouis_liblouisutdml
+ * Class:     org.liblouis.LibLouis
+ * Method:    setLogFile
+ * Signature: (Ljava/lang/String)V
+ */
+JNIEXPORT void JNICALL Java_org_liblouis_LibLouis_setLogFile
+  (JNIEnv * env, jobject obj, jstring fileName)
+{
+  const char *logf = NULL;
+  logf = (*env)->GetStringUTFChars(env, fileName, NULL);
+  if (logf == NULL)
+    return;
+  lou_logFile(logf);
+  (*env)->ReleaseStringUTFChars(env, fileName, logf);
+}
+
+/*
+ * Class:     org.liblouis.LibLouis
+ * Method:    logEnd
+ * Signature: ()V
+ */
+JNIEXPORT void JNICALL Java_org_liblouis_LibLouis_logEnd
+  (JNIEnv * env, jobject obj)
+{
+  lou_logEnd();
+}
+
+/*
+ * Class:     org_liblouis_LibLouis
  * Method:    charSize
  * Signature: ()I
  */
-JNIEXPORT jint JNICALL Java_org_liblouis_liblouisutdml_charSize
+JNIEXPORT jint JNICALL Java_org_liblouis_LibLouis_charSize
   (JNIEnv * env, jobject this)
 {
   return CHARSIZE;
 }
 
 /*
- * Class:     org_liblouis_liblouisutdml
+ * Class:     org.liblouis.LibLouis
+ * Method:    setLogLevel
+ * Signature: (I)V
+ */
+JNIEXPORT void JNICALL Java_org_liblouis_LibLouis_setLogLevel
+  (JNIEnv * env, jobject this, jint level)
+{
+  lou_setLogLevel(level);
+}
+
+static jobject louLogCBFunc;
+static void javaLouLogCallback(int level, const char *message)
+{
+  execJavaLogCallback(louLogCBFunc, level, message);
+}
+/*
+ * Class:     org.liblouis.LibLouis
+ * Method:    registerLogCallback
+ * Signature: (Lorg/liblouis/LogCallback)V
+ */
+JNIEXPORT void JNICALL Java_org_liblouis_LibLouis_registerLogCallback
+  (JNIEnv * env, jobject this, jobject cb)
+{
+  // Ensure we have a reference to the JVM
+  if (jvm == NULL)
+  {
+    jint rs = (*env)->GetJavaVM(env, &jvm);
+    if (rs != JNI_OK)
+      return;
+  }
+  // Remove existing references to the callback
+  if (louLogCBFunc != NULL)
+  {
+    (*env)->DeleteGlobalRef(env, louLogCBFunc);
+    louLogCBFunc = NULL;
+  }
+  // Now set the callback according to what is passed in cb
+  if (cb != NULL)
+  {
+    louLogCBFunc = (*env)->NewGlobalRef(env, cb);
+  }
+  if (louLogCBFunc != NULL)
+  {
+    lou_registerLogCallback(javaLouLogCallback);
+  }
+  else
+  {
+    lou_registerLogCallback(NULL);
+  }
+}
+/*
+ * Class:     org_liblouis_LibLouisUTDML
  * Method:    setWriteablePath
  * Signature: (Ljava/lang/String;)V
  */
-JNIEXPORT void JNICALL Java_org_liblouis_liblouisutdml_setWriteablePath
+JNIEXPORT void JNICALL Java_org_liblouis_LibLouisUTDML_setWriteablePath
   (JNIEnv * env, jclass obj, jstring path)
 {
   const jbyte *pathX = NULL;
@@ -634,11 +761,11 @@ release:
 }
 
 /*
- * Class:     org_liblouis_liblouisutdml
+ * Class:     org_liblouis_LibLouisUTDML
  * Method:    free
  * Signature: ()V
  */
-JNIEXPORT void JNICALL Java_org_liblouis_liblouisutdml_free
+JNIEXPORT void JNICALL Java_org_liblouis_LibLouisUTDML_free
   (JNIEnv * env, jobject this)
 {
   lbu_free ();
@@ -646,7 +773,7 @@ JNIEXPORT void JNICALL Java_org_liblouis_liblouisutdml_free
 }
 
 /*
- * Class:     org_liblouis_liblouisutdml
+ * Class:     org_liblouis_LibLouisUTDML
  * Method:    file2brl
  * Signature: ([Ljava/lang/String;)Z
  */
@@ -676,7 +803,7 @@ getArg (JNIEnv * env, jobject obj, jobjectArray args, jint index)
   return curArg;
 }
 
-JNIEXPORT jboolean JNICALL Java_org_liblouis_liblouisutdml_file2brl
+JNIEXPORT jboolean JNICALL Java_org_liblouis_LibLouisUTDML_file2brl
   (JNIEnv * env, jobject obj, jobjectArray args)
 {
   jint numArgs = (*env)->GetArrayLength (env, args);
@@ -749,7 +876,7 @@ JNIEXPORT jboolean JNICALL Java_org_liblouis_liblouisutdml_file2brl
 		  k++;
 		  break;
 		default:
-		  lou_logPrint ("invalid argument '%s'", curArg);
+		  logMessage (LOG_ERROR, "invalid argument '%s'", curArg);
 		  return JNI_FALSE;
 		}
 	      continue;
@@ -769,7 +896,7 @@ JNIEXPORT jboolean JNICALL Java_org_liblouis_liblouisutdml_file2brl
 		}
 	      else
 		{
-		  lou_logPrint ("extra operand: '%s'\n",
+		  logMessage (LOG_ERROR, "extra operand: '%s'\n",
 				getArg (env, obj, args, k + 2));
 		  return JNI_FALSE;
 		}
@@ -779,7 +906,7 @@ JNIEXPORT jboolean JNICALL Java_org_liblouis_liblouisutdml_file2brl
       getArg (env, obj, args, -1);
     }
   if (logFileName[0] != 0)
-    lou_logFile (logFileName);
+    lbu_logFile (logFileName);
   if (whichProc == 0)
     whichProc = 'x';
   if (configSettings != NULL)
@@ -789,15 +916,15 @@ JNIEXPORT jboolean JNICALL Java_org_liblouis_liblouisutdml_file2brl
   if (!read_configuration_file  (configFileList, logFileName, 
 configSettings, 0))
     {
-      lou_logEnd ();
+      lbu_logEnd ();
       return JNI_FALSE;
     }
   if (strcmp (inputFileName, "stdin") != 0)
     {
       if (!(inputFile = fopen (inputFileName, "rb")))
 	{
-	  lou_logPrint ("Can't open input file '%s'.\n", inputFileName);
-	  lou_logEnd ();
+	  logMessage (LOG_ERROR, "Can't open input file '%s'.\n", inputFileName);
+	  lbu_logEnd ();
 	  return JNI_FALSE;
 	}
     }
@@ -808,8 +935,8 @@ configSettings, 0))
   strcat (tempFileName, "file2brl.temp");
   if (!(tempFile = fopen (tempFileName, "wb")))
     {
-      lou_logPrint ("Can't open temporary file.\n");
-      lou_logEnd ();
+      logMessage (LOG_ERROR, "Can't open temporary file.\n");
+      lbu_logEnd ();
       return JNI_FALSE;
     }
   if (whichProc == 'p')
@@ -903,7 +1030,7 @@ configSettings, 0))
 	       (configFileList, tempFileName, temp2FileName, NULL,
 		NULL, mode)) != 1)
 	    {
-	      lou_logEnd ();
+	      lbu_logEnd ();
 	      return JNI_FALSE;
 	    }
 	  if (ud->back_text == html)
@@ -926,12 +1053,12 @@ configSettings, 0))
 			   NULL, mode);
 	break;
       default:
-	lou_logPrint ("Program bug %c\n", whichProc);
+	logMessage (LOG_ERROR, "Program bug %c\n", whichProc);
 	break;
       }
   if (configSettings != NULL)
     free (configSettings);
-  lou_logEnd ();
+  lbu_logEnd ();
   return JNI_TRUE;
 }
 
@@ -1056,12 +1183,12 @@ release:
 }
 
 /*
- * Class:     org_liblouis_liblouisutdml
- * Method:    louisTranslateString
+ * Class:     org_liblouis_LibLouis
+ * Method:    translateString
  * Signature: (Ljava/lang/String;[B[I[B[I[BLjava/lang/String;I)Z
  */
 JNIEXPORT jboolean JNICALL
-Java_org_liblouis_liblouisutdml_louisTranslateString (JNIEnv * env,
+Java_org_liblouis_LibLouis_translateString (JNIEnv * env,
 						      jobject obj,
 						      jstring tableList,
 						      jbyteArray inbuf,
@@ -1077,11 +1204,11 @@ Java_org_liblouis_liblouisutdml_louisTranslateString (JNIEnv * env,
 }
 
 /*
- * Class:     org_liblouis_liblouisutdml
- * Method:    louisTranslate
+ * Class:     org_liblouis_LibLouis
+ * Method:    translate
  * Signature: (Ljava/lang/String;[B[I[B[I[B[I[I[ILjava/lang/String;I)Z
  */
-JNIEXPORT jboolean JNICALL Java_org_liblouis_liblouisutdml_louisTranslate
+JNIEXPORT jboolean JNICALL Java_org_liblouis_LibLouis_translate
   (JNIEnv * env, jobject obj, jstring tableList, jbyteArray inbuf,
    jintArray inlen,
    jbyteArray outbuf, jintArray outlen,
@@ -1094,11 +1221,11 @@ JNIEXPORT jboolean JNICALL Java_org_liblouis_liblouisutdml_louisTranslate
 }
 
 /*
- * Class:     org_liblouis_liblouisutdml
+ * Class:     org_liblouis_LibLouis
  * Method:    hyphenate
  * Signature: (Ljava/lang/String;[BI[BLjava/lang/String;I)Z
  */
-JNIEXPORT jboolean JNICALL Java_org_liblouis_liblouisutdml_hyphenate
+JNIEXPORT jboolean JNICALL Java_org_liblouis_LibLouis_hyphenate
   (JNIEnv * env, jobject obj, jstring tableList, jbyteArray inbuf, jint
    inlen, jbyteArray hyphens, jstring logFile, jint mode)
 {
@@ -1149,12 +1276,12 @@ release:
 }
 
 /*
- * Class:     org_liblouis_liblouisutdml
- * Method:    louisBackTranslateString
+ * Class:     org_liblouis_LibLouis
+ * Method:    backTranslateString
  * Signature: (Ljava/lang/String;[B[I[B[I[BLjava/lang/String;I)Z
  */
 JNIEXPORT jboolean JNICALL
-  Java_org_liblouis_liblouisutdml_louisBackTranslateString
+  Java_org_liblouis_LibLouis_backTranslateString
   (JNIEnv * env, jobject obj,
    jstring tableList,
    jbyteArray
@@ -1169,11 +1296,11 @@ JNIEXPORT jboolean JNICALL
 }
 
 /*
- * Class:     org_liblouis_liblouisutdml
- * Method:    louisBackTranslate
+ * Class:     org_liblouis_LibLouis
+ * Method:    backTranslate
  * Signature: (Ljava/lang/String;[B[I[B[I[B[I[I[ILjava/lang/String;I)Z
  */
-JNIEXPORT jboolean JNICALL Java_org_liblouis_liblouisutdml_louisBackTranslate
+JNIEXPORT jboolean JNICALL Java_org_liblouis_LibLouis_backTranslate
   (JNIEnv * env, jobject obj, jstring tableList, jbyteArray inbuf,
    jintArray inlen,
    jbyteArray outbuf, jintArray outlen,
@@ -1186,44 +1313,82 @@ JNIEXPORT jboolean JNICALL Java_org_liblouis_liblouisutdml_louisBackTranslate
 }
 
 /*
- * Class:     org_liblouis_liblouisutdml
+ * Class:     org_liblouis_LibLouisUTDML
  * Method:    setLogFile
  * Signature: (Ljava/lang/String;)V
  */
-JNIEXPORT void JNICALL Java_org_liblouis_liblouisutdml_setLogFile
+JNIEXPORT void JNICALL Java_org_liblouis_LibLouisUTDML_setLogFile
   (JNIEnv * env, jobject obj, jstring logFile)
 {
-  const jbyte *logf = NULL;
+  const char *logf = NULL;
   logf = (*env)->GetStringUTFChars (env, logFile, NULL);
   if (logf == NULL)
     return;
-  lou_logFile (logf);
+  lbu_logFile (logf);
   (*env)->ReleaseStringUTFChars (env, logFile, logf);
 }
 
 /*
- * Class:     org_liblouis_liblouisutdml
- * Method:    logMessage
- * Signature: (Ljava/lang/String;)V
- */
-JNIEXPORT void JNICALL Java_org_liblouis_liblouisutdml_logMessage
-  (JNIEnv * env, jobject obj, jstring message)
-{
-  const jbyte *messagex = NULL;
-  messagex = (*env)->GetStringUTFChars (env, message, NULL);
-  if (messagex == NULL)
-    return;
-  lou_logPrint (messagex);
-  (*env)->ReleaseStringUTFChars (env, message, messagex);
-}
-
-/*
- * Class:     org_liblouis_liblouisutdml
+ * Class:     org_liblouis_LibLouisUTDML
  * Method:    logEnd
  * Signature: ()V
  */
-JNIEXPORT void JNICALL Java_org_liblouis_liblouisutdml_logEnd
+JNIEXPORT void JNICALL Java_org_liblouis_LibLouisUTDML_logEnd
   (JNIEnv * env, jobject this)
 {
-  lou_logEnd ();
+  lbu_logEnd ();
+}
+
+static jobject lbuLogCBFunc;
+static void javaLbuLogCallbackFunc(int level, const char *message)
+{
+  execJavaLogCallback(lbuLogCBFunc, level, message);
+}
+
+/*
+ * Class:     org_liblouis_LibLouisUTDML
+ * Method:    registerLogCallback
+ * Signature: (Lorg/liblouis/LogCallback;)V
+ */
+JNIEXPORT void JNICALL Java_org_liblouis_LibLouisUTDML_registerLogCallback
+  (JNIEnv * env, jobject this, jobject cb)
+{
+  // if not previously set, set the JVM pointer
+  if (jvm == NULL)
+  {
+    jint rs = (*env)->GetJavaVM(env, &jvm);
+    if (rs != JNI_OK)
+    {
+      return;
+    }
+  }
+  // Remove any existing global reference to callbacks
+  if (lbuLogCBFunc != NULL)
+  {
+    (*env)->DeleteGlobalRef(env, lbuLogCBFunc);
+    lbuLogCBFunc = NULL;
+  }
+  if (cb != NULL)
+  {
+    lbuLogCBFunc = (*env)->NewGlobalRef(env, cb);
+  }
+  if (lbuLogCBFunc != NULL)
+  {
+    lbu_registerLogCallback(javaLbuLogCallbackFunc);
+  }
+  else
+  {
+    lbu_registerLogCallback(NULL);
+  }
+}
+
+/*
+ * Class:    org_liblouis_LibLouisUTDML
+ * Method:   setLogLevel
+ *Signature: (I)V
+ */
+JNIEXPORT void JNICALL Java_org_liblouis_LibLouisUTDML_setLogLevel
+  (JNIEnv * env, jobject this, jint level)
+{
+  lbu_setLogLevel(level);
 }
