@@ -35,10 +35,11 @@
 #include <string.h>
 #include "louisutdml.h"
 #include <libxml/HTMLparser.h>
+#include <libxml/catalog.h>
 
 UserData *ud = NULL;
 
-char *EXPORT_CALL
+LBUAPI char *EXPORT_CALL
 lbu_version ()
 {
   static char *version = PACKAGE_VERSION;
@@ -49,6 +50,10 @@ lbu_version ()
   return bothVersions;
 }
 
+LBUAPI void EXPORT_CALL lbu_loadXMLCatalog(const char *filename)
+{
+  xmlLoadCatalog(filename);
+}
 void
 libxml_errors (void *ctx ATTRIBUTE_UNUSED, const char *msg, ...)
 {
@@ -58,7 +63,7 @@ libxml_errors (void *ctx ATTRIBUTE_UNUSED, const char *msg, ...)
   memset (buffer, 0, sizeof (buffer));
   vsnprintf (buffer, sizeof (buffer) - 4, msg, args);
   va_end (args);
-  lou_logPrint ("%s", buffer);
+  logMessage (LOG_ERROR, "%s", buffer);
 }
 
 static xmlParserCtxt *ctxt;
@@ -80,7 +85,7 @@ cleanupLibxml ()
 static void
 freeEverything ()
 {
-  lou_logEnd ();
+  lbu_logEnd ();
   cleanupLibxml ();
   lbu_free ();
 }
@@ -119,8 +124,8 @@ processXmlDocument (const char *inputDoc, int length)
 	    ud->doc = xmlParseFile (inputDoc);
 	  if (ud->doc == NULL)
 	    {
-	      lou_logPrint
-		("Document could not be processed; may be \
+	      logMessage
+		(LOG_FATAL, "Document could not be processed; may be \
 malformed or contain illegal characters");
 	      cleanupLibxml ();
 	      return 0;
@@ -131,20 +136,20 @@ malformed or contain illegal characters");
     ud->doc = xmlParseMemory (inputDoc, length);
   if (ud->doc == NULL)
     {
-      lou_logPrint ("Document could not be processed, probably  malformed");
+      logMessage (LOG_FATAL, "Document could not be processed, probably  malformed");
       cleanupLibxml ();
       return 0;
     }
   if (ud->doc->encoding == NULL)
     {
-      lou_logPrint ("Encoding, preferably UTF-8,  must be specified");
+      logMessage (LOG_ERROR, "Encoding, preferably UTF-8,  must be specified");
       cleanupLibxml ();
       return 0;
     }
   if (ud->format_for >= utd && ignore_case_comp (ud->doc->encoding,
 						 "UTF-8", 5) != 0)
     {
-      lou_logPrint ("UTDML requires UTF-8 encoding, not '%s'",
+      logMessage (LOG_ERROR, "UTDML requires UTF-8 encoding, not '%s'",
 		    ud->doc->encoding);
       cleanupLibxml ();
       return 0;
@@ -152,7 +157,7 @@ malformed or contain illegal characters");
   rootElement = xmlDocGetRootElement (ud->doc);
   if (rootElement == NULL)
     {
-      lou_logPrint ("Document is empty");
+      logMessage (LOG_ERROR, "Document is empty");
       cleanupLibxml ();
       return 0;
     }
@@ -174,7 +179,7 @@ malformed or contain illegal characters");
 	}
       if (!transcribe_document (rootElement))
 	{
-	  lou_logPrint ("Document could not be transcribed");
+	  logMessage (LOG_ERROR, "Document could not be transcribed");
 	  cleanupLibxml ();
 	  return 0;
 	}
@@ -183,7 +188,7 @@ malformed or contain illegal characters");
   return 1;
 }
 
-void *EXPORT_CALL
+LBUAPI void *EXPORT_CALL
 lbu_initialize (const char *configFileList,
 		const char *logFileName, const char *settingsString)
 {
@@ -193,7 +198,7 @@ lbu_initialize (const char *configFileList,
   return (void *) ud;
 }
 
-int EXPORT_CALL
+LBUAPI int EXPORT_CALL
 lbu_translateString (const char *configFileList,
 		     const char *inbuf, int inlen, widechar * outbuf,
 		     int *outlen,
@@ -225,7 +230,7 @@ lbu_translateString (const char *configFileList,
       else
 	k = transcribe_text_string ();
       *outlen = ud->outlen_so_far;
-      lou_logEnd ();
+      lbu_logEnd ();
       return k;
     }
   if (inbuf[k + 1] == '?')
@@ -235,7 +240,7 @@ lbu_translateString (const char *configFileList,
       inlen += strlen (ud->xml_header);
       if (!(xmlInbuf = malloc (inlen + 4)))
 	{
-	  lou_logPrint ("Not enough memory");
+	  logMessage (LOG_FATAL, "Not enough memory");
 	  return 0;
 	}
       strcpy (xmlInbuf, ud->xml_header);
@@ -246,12 +251,11 @@ lbu_translateString (const char *configFileList,
   *outlen = ud->outlen_so_far;
   if (xmlInbuf != inbuf)
     free (xmlInbuf);
-  lou_logEnd ();
+  lbu_logEnd ();
   return k;
 }
 
-int
-  EXPORT_CALL lbu_translateFile
+LBUAPI int EXPORT_CALL lbu_translateFile
   (const char *configFileList, const char *inFileName,
    const char *outFileName, const char *logFileName,
    const char *settingsString, unsigned int mode)
@@ -270,7 +274,7 @@ int
     {
       if (!(ud->outFile = fopen (outFileName, "wb")))
 	{
-	  lou_logPrint ("Can't open output file %s.", outFileName);
+	  logMessage (LOG_ERROR, "Can't open output file %s.", outFileName);
 	  return 0;
 	}
     }
@@ -279,12 +283,11 @@ int
   k = processXmlDocument (inFileName, 0);
   if (ud->outFile != stdout)
     fclose (ud->outFile);
-  lou_logEnd ();
+  lbu_logEnd ();
   return k;
 }
 
-int
-  EXPORT_CALL lbu_translateTextFile
+LBUAPI int EXPORT_CALL lbu_translateTextFile
   (const char *configFileList, const char *inFileName,
    const char *outFileName, const char *logFileName,
    const char *settingsString, unsigned int mode)
@@ -305,7 +308,7 @@ int
     {
       if (!(ud->inFile = fopen (inFileName, "rb")))
 	{
-	  lou_logPrint ("Can't open input file %s.", inFileName);
+	  logMessage (LOG_ERROR, "Can't open input file %s.", inFileName);
 	  return 0;
 	}
     }
@@ -315,7 +318,7 @@ int
     {
       if (!(ud->outFile = fopen (outFileName, "wb")))
 	{
-	  lou_logPrint ("Can't open output file %s.", outFileName);
+	  logMessage (LOG_ERROR, "Can't open output file %s.", outFileName);
 	  return 0;
 	}
     }
@@ -334,11 +337,11 @@ int
     fclose (ud->inFile);
   if (ud->outFile != stdout)
     fclose (ud->outFile);
-  lou_logEnd ();
+  lbu_logEnd ();
   return k;
 }
 
-int EXPORT_CALL
+LBUAPI int EXPORT_CALL
 lbu_backTranslateString (const char *configFileList,
 			 const char *inbuf, int inlen, widechar
 			 * outbuf,
@@ -347,6 +350,7 @@ lbu_backTranslateString (const char *configFileList,
 			 *settingsString, unsigned int mode)
 {
   int k;
+  logMessage(LOG_INFO, "Begin lbu_backTranslateString: inbuf=%s", inbuf);
   if (!read_configuration_file
       (configFileList, logFileName, settingsString, mode))
     return 0;
@@ -358,7 +362,10 @@ lbu_backTranslateString (const char *configFileList,
   ud->outlen = *outlen;
   ud->inFile = ud->outFile = NULL;
   if (ud->format_for == utd)
+  {
+    logMessage(LOG_DEBUG, "ud->format_for=utd");
     k = utd_back_translate_braille_string ();
+  }
   else
     k = back_translate_braille_string ();
   if (!k)
@@ -367,12 +374,12 @@ lbu_backTranslateString (const char *configFileList,
       return 0;
     }
   *outlen = ud->outlen_so_far;
-  lou_logEnd ();
+  logMessage(LOG_INFO, "Finish lbu_backTranslateString");
+  lbu_logEnd ();
   return 1;
 }
 
-int
-  EXPORT_CALL lbu_backTranslateFile
+LBUAPI int EXPORT_CALL lbu_backTranslateFile
   (const char *configFileList, const char *inFileName,
    const char *outFileName, const char *logFileName,
    const char *settingsString, unsigned int mode)
@@ -394,7 +401,7 @@ int
     {
       if (!(ud->inFile = fopen (inFileName, "rb")))
 	{
-	  lou_logPrint ("Can't open input file %s.", inFileName);
+	  logMessage (LOG_ERROR, "Can't open input file %s.", inFileName);
 	  return 0;
 	}
     }
@@ -404,7 +411,7 @@ int
     {
       if (!(ud->outFile = fopen (outFileName, "wb")))
 	{
-	  lou_logPrint ("Can't open output file %s.", outFileName);
+	  logMessage (LOG_ERROR, "Can't open output file %s.", outFileName);
 	  return 0;
 	}
     }
@@ -423,14 +430,14 @@ int
     fclose (ud->inFile);
   if (ud->outFile != stdout)
     fclose (ud->outFile);
-  lou_logEnd ();
+  lbu_logEnd ();
   return 1;
 }
 
 static char writeablePath[MAXNAMELEN];
 static char *writeablePathPtr = NULL;
 
-char *EXPORT_CALL
+LBUAPI char *EXPORT_CALL
 lbu_setWriteablePath (const char *path)
 {
   writeablePathPtr = NULL;
@@ -441,13 +448,13 @@ lbu_setWriteablePath (const char *path)
   return writeablePathPtr;
 }
 
-char *EXPORT_CALL
+LBUAPI char *EXPORT_CALL
 lbu_getWriteablePath ()
 {
   return writeablePathPtr;
 }
 
-int EXPORT_CALL
+LBUAPI int EXPORT_CALL
 lbu_charToDots (const char *tableList, const unsigned char *inbuf,
 		unsigned char *outbuf, int length, const char *logFile,
 		unsigned int mode)
@@ -458,7 +465,7 @@ lbu_charToDots (const char *tableList, const unsigned char *inbuf,
   int result = 0;
   if (tableList == NULL || inbuf == NULL || outbuf == NULL)
     return 0;
-  lou_logFile (logFile);
+  lbu_logFile (logFile);
   interBuf = malloc (length * CHARSIZE);
   utf8Length = length;
   wcLength = length;
@@ -470,12 +477,12 @@ lbu_charToDots (const char *tableList, const unsigned char *inbuf,
       utf8Length = length;
       wc_string_to_utf8 (interBuf, &wcLength, outbuf, &utf8Length);
     }
-  lou_logEnd ();
+  lbu_logEnd ();
   free (interBuf);
   return result;
 }
 
-int EXPORT_CALL
+LBUAPI int EXPORT_CALL
 lbu_dotsToChar (const char *tableList, const unsigned char *inbuf,
 		unsigned char *outbuf, int length, const char *logFile,
 		unsigned int mode)
@@ -486,7 +493,7 @@ lbu_dotsToChar (const char *tableList, const unsigned char *inbuf,
   int result = 0;
   if (tableList == NULL || inbuf == NULL || outbuf == NULL)
     return 0;
-  lou_logFile (logFile);
+  lbu_logFile (logFile);
   interBuf = malloc (length * CHARSIZE);
   utf8Length = length;
   wcLength = length;
@@ -497,28 +504,28 @@ lbu_dotsToChar (const char *tableList, const unsigned char *inbuf,
       utf8Length = length;
       wc_string_to_utf8 (interBuf, &wcLength, outbuf, &utf8Length);
     }
-  lou_logEnd ();
+  lbu_logEnd ();
   free (interBuf);
   return result;
 }
 
-int EXPORT_CALL
+LBUAPI int EXPORT_CALL
 lbu_checkTable (const char *tableList, const char *logFile, unsigned int mode)
 {
   int result = 1;
-  lou_logFile (logFile);
+  lbu_logFile (logFile);
   if (!lou_getTable (tableList))
     result = 0;
-  lou_logEnd ();
+  lbu_logEnd ();
   return result;
 }
 
-void EXPORT_CALL
+LBUAPI void EXPORT_CALL
 lbu_free ()
 {
 /* Free all memory used by liblouisutdml. You MUST call this function at 
 * the END of your application.*/
-  lou_logEnd ();
+  lbu_logEnd ();
   lou_free ();
   destroy_semantic_table ();
   if (ud != NULL)
