@@ -7,7 +7,7 @@
    Copyright (C) 2004, 2005, 2006
    ViewPlus Technologies, Inc. www.viewplus.com
    and
-   Abilitiessoft, Inc. www.abilitiessoft.com
+   Abilitiessoft, Inc. www.abilitiessoft.org
    All rights reserved
 
    This file is free software; you can redistribute it and/or modify it
@@ -26,7 +26,7 @@
    the Free Software Foundation, 51 Franklin Street, Fifth Floor,
    Boston, MA 02110-1301, USA.
 
-   Maintained by John J. Boyer john.boyer@abilitiessoft.com
+   Maintained by John J. Boyer john.boyer@abilitiessoft.org
    */
 
 #include <stdio.h>
@@ -99,7 +99,7 @@ initialize_contents (void)
   strcat (tempFileName, "lbx_body.temp");
   if (!(tempFile = fopen (tempFileName, "w")))
     {
-      lou_logPrint ("Can't open temporary file.\n");
+      logMessage (LOG_ERROR, "Can't open temporary file.\n");
       return 0;
     }
   ud->outFile = tempFile;
@@ -282,7 +282,7 @@ make_contents (void)
 
   if (!(tempFile = fopen (tempFileName, "r")))
     {
-      lou_logPrint ("Can't open temporary file.\n");
+      logMessage (LOG_ERROR, "Can't open temporary file.\n");
       return 0;
     }
   do
