@@ -32,8 +32,10 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include <ctype.h>
 #include "louisutdml.h"
 #include <libxml/xpath.h>
+#include <libxml/xpathInternals.h>
 #include "sem_names.h"
 
 typedef struct
