@@ -39,6 +39,7 @@ transcribe_document (xmlNode * node)
   StyleType *style;
   xmlNode *child;
   int childrenDone = 0;
+  logMessage(LOG_DEBUG, "Begin transcribe_document");
   ud->top = -1;
   ud->style_top = -1;
   ud->text_length = 0;
@@ -53,6 +54,8 @@ transcribe_document (xmlNode * node)
   switch (ud->stack[ud->top])
     {
     case no:
+      if (ud->format_for == utd)
+        break;
       if (ud->text_length > 0 && ud->text_length < MAX_LENGTH &&
 	  ud->text_buffer[ud->text_length - 1] > 32)
 	ud->text_buffer[ud->text_length++] = 32;
@@ -141,5 +144,6 @@ transcribe_document (xmlNode * node)
     end_style ();
   end_document ();
   pop_sem_stack ();
+  logMessage(LOG_DEBUG, "Finished transcribe_document");
   return 1;
 }
