@@ -42,6 +42,7 @@ static const struct option longopts[] = {
   {"poorly-formatted", no_argument, NULL, 'p'},
   {"html", no_argument, NULL, 't'},
   {"text", no_argument, NULL, 'T'},
+  {"paragraph-line", no_argument, NULL, 'P'},
   {"log-file", no_argument, NULL, 'l'},
   {"config-setting", required_argument, NULL, 'C'},
   {"writeable-path", required_argument, NULL, 'w'},
@@ -76,13 +77,16 @@ is not specified the output is sent to stdout.\n\n", stdout);
   -b, --backward      	  backward translation\n\
   -r, --reformat      	  reformat a braille file\n\
   -T, --text		  Treat as text even if xml\n\
-  -p, --poorly-formatted  translate a poorly formatted file\n\
   -t, --html              html document, not xhtml\n\
+  -p, --poorly-formatted  translate a poorly formatted file\n\
+  -P, --paragraph-line    treat each block of text ending in a newline\n\
+                          as a paragraph. If there are two newline characters\n\
+                          a blank line will be inserted before the next paragraph\n\
   -C, --config-setting    specify particular configuration settings\n\
                           They override any settings that are specified in a\n\
                           config file\n\
-  -w  --writeable-path   path for temp files and log file\n\
-  -l, --log-file          write errors to file2brl.log instead of stderr\n", 
+  -w  --writeable-path    path for temp files and log file\n\
+  -l, --log-file          write errors to file2brl.log instead of stderr\n",
   stdout);
 
   printf ("\n");
@@ -111,11 +115,13 @@ main (int argc, char **argv)
   UserData *ud;
 
   int optc;
+  lbu_setLogLevel(LOG_DEBUG);
+  lou_setLogLevel(LOG_DEBUG);
   set_program_name (argv[0]);
   logFileName[0] = 0;
   
   while ((optc =
-	  getopt_long (argc, argv, "hvf:brptlw:TC:", longopts, NULL)) != 
+	  getopt_long (argc, argv, "hvf:brpPtlw:TC:", longopts, NULL)) != 
 	  -1)
     switch (optc)
       {
@@ -140,6 +146,7 @@ main (int argc, char **argv)
 	break;
       case 'b':
       case 'p':
+      case 'P':
       case 'r':
       case 'T':
       case '0':
@@ -206,7 +213,7 @@ main (int argc, char **argv)
     {
       if (!(inputFile = fopen (inputFileName, "r")))
 	{
-	  lou_logPrint ("Can't open input file %s.\n", inputFileName);
+	  logMessage (LOG_FATAL, "Can't open input file %s.\n", inputFileName);
 	  exit (EXIT_FAILURE);
 	}
     }
@@ -217,7 +224,7 @@ main (int argc, char **argv)
   strcat (tempFileName, "file2brl.temp");
   if (!(tempFile = fopen (tempFileName, "w")))
     {
-      lou_logPrint ("Can't open temporary file.\n");
+      logMessage (LOG_FATAL, "Can't open temporary file.\n");
       exit (EXIT_FAILURE);
     }
   if (whichProc == 'p')
@@ -283,6 +290,8 @@ main (int argc, char **argv)
 	  }
 	if (pch == '>' && ch == '<')
 	  fputc (10, tempFile);
+	if (whichProc == 'P' && ch == 10 && pch != 10)
+	  fputc (10, tempFile);
 	fputc (ch, tempFile);
 	pch = ch;
 	charsRead++;
@@ -344,7 +353,7 @@ main (int argc, char **argv)
 	  exit (EXIT_FAILURE);
 	break;
       default:
-	lou_logPrint ("Program bug %c\n", whichProc);
+	logMessage (LOG_FATAL, "Program bug %c\n", whichProc);
 	break;
       }
   lbu_free ();
